/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *									     *
 * Library:	libsubdiv2.5D						     *
 *									     *
 * Header:	subdiv2.5D.h						     *
 *									     *
 * Description:	public interface to library				     *
 *									     *
 * Date:	20000313						     *
 *									     *
 * Author:	Francisco MORN BURGOS   (http://www.gti.ssr.upm.es./~fmb/)  *
 *		Grupo Tratamiento Imgenes, Univ. Politcnica Madrid, Spain  *
 *									     *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



#ifndef _INC_SUBDIV25D_
#define _INC_SUBDIV25D_



#include <stdio.h>



/* * * * *     CONSTANTS & TYPES     * * * * */

#ifdef FALSE
#undef FALSE
#endif
#ifdef TRUE
#undef TRUE
#endif
#define FALSE	0
#define TRUE	!FALSE
typedef int sdBoolean;	/* yep, 31 bits wasted manywhere... */

/* leave the definition of "HRD" (High Resolution Details) below if you	*/
/* want 32 bits/detail; comment it out to use only 16 bits/detail	*/
#define HRD
#ifdef HRD
#  define CODEWORD	unsigned int
#  define NBPD		32
#  define NBPDN		12		/* 1+11 bits for coordinate N */
#  define NBPDT		10		/* 1+ 9 bits for coordinate T{1,2} */
#  define MASK_SN	0x80000000
#  define MASK_ST1	0x00080000
#  define MASK_ST2	0x00000200
#else
#  define CODEWORD	unsigned short
#  define NBPD		16
#  define NBPDN		 8		/* 1+ 7 bits for coordinate N */
#  define NBPDT		 4		/* 1+ 3 bits for coordinate T{1,2} */
#  define MASK_SN	0x8000
#  define MASK_ST1	0x0080
#  define MASK_ST2	0x0008
#endif

typedef enum {OWN, DAVE, TECPLOT, SNHC, RVRML2, GUESS} sdInputFormat;
typedef enum {USELESS, MATHEMATICA, VRML2} sdOutputFormat;
typedef enum {BUTTERFLY, MIDPOINT, SPHERE} sdScheme;

typedef struct _V	*sdVectorPtr;
typedef struct _VLN	*sdVectorLst;
typedef struct _P	*sdPointPtr;
typedef struct _P	*sdPointLst;
typedef struct _OTN	*sdOctree;
typedef struct _T	*sdTrianglePtr;
typedef struct _TLN	*sdTriangleLst;
typedef struct _DTN	*sdDetailTree;
typedef struct _DTLN	*sdDTreeList;
typedef struct _E	*sdEdgePtr;
typedef struct _M	*sdMeshPtr;

typedef struct _V {
  float		x, y, z;
} sdVector;

typedef struct _VLN {
  sdVector	  pos;
  sdVectorLst	  next;
} sdVectorLstNode;

typedef struct _P {
  int		n;
  sdPointPtr	next,
		prev;
  sdVector	pos,
		delta,
		normal;		/* normal binding can be "per vertex"... */
  CODEWORD	codedelta;
  sdTrianglePtr	t;
} sdPoint;

typedef struct _OTN {
  int		depth;
  sdBoolean	tree;
  sdOctree	*son,	/* will be "son[8]" once allocated */
		mom;
  sdPointPtr	pt;
} sdOctreeNode;

typedef struct _T {
  int		n,
		depth;
  sdBoolean	subdiv,
		inv;
  sdTrianglePtr	adt[3],	/* 3 (one per edge) => only manifold meshes */
		*son,	/* will be "son[4]" once allocated */
		mom;
  sdPointPtr	vtx[3];
  sdDetailTree	dt;
  sdVector	normal;		/* ... and normal binding can be "per face" */
  float		K;	/* will be area or n.v0 */
} sdTriangle;

typedef struct _TLN {
  sdTrianglePtr	t;
  sdTriangleLst	next;
} sdTriangleLstNode;

typedef struct _DTN {
  char		*id;
  sdDetailTree	*son;	/* will be "son[4]" once allocated */
  sdPointPtr	pt;
  CODEWORD	D, L;
} sdDetailTreeNode;

typedef struct _DTLN {
  sdDetailTree	dt;
  int		n;
  sdDTreeList	next,
		prev;
} sdDTreeListNode;

typedef struct _E {
  sdPointPtr	vtx[2];
  sdTrianglePtr	adt[2];	/* 2 => only manifold meshes */
  int		e4t[2];	/* 2 => only manifold meshes */
  sdBoolean	highlighted;
} sdEdge;

typedef struct _M {
  int		verbose;
  sdScheme	subdivScheme;
  sdBoolean	nonManifold,
		closed,
		conforming,
		normalsPerVertex,
		display,
		refresh,
		progressive,
		fDetailsIn,
		fDetailsBinary,
		useDiMePlugin,
		useBarycentre,
		partition;
  FILE		*fDetails;
  sdVector	barycentre;
  float		scale,
		epsilon,
		maxDetailN,
		maxDetailT;
  int		Npoints,
		IDpoint,
		NnodesOT;	/* not really needed */
  sdOctree	vertexOT;
  sdPointPtr	firstVertex,
		lastVertex,
		pickedVertex;
  int		NmotherTriangles,
		NleafTriangles,
		Ntriangles,
		IDtriangle;
  sdTrianglePtr	*motherTriangle;
  sdTriangleLst tmpTriangleList;
  int		depth,
		targetDepth;
  sdDetailTree	*motherDetailTree;
  sdMeshPtr	targetMesh;
  sdPointPtr	*targetMeshVtx;
  int		Nedges,
		NmaxNestedCalls2FPBPIP;
  sdEdge	*edge;
  sdVectorLst	*targetMeshPath4edge;
} sdMesh;



/* * * * *     FUNCTION PROTOTYPES     * * * * */

/* from "subdiv2.5D_gfx.c" */
extern void sdDisplayMeshForever(sdMeshPtr m,
				 int width, int height, char *title);

/* from "subdiv2.5D_io.c" */
extern void sdReadInitMesh(sdMeshPtr m, char *filename, sdInputFormat format);
extern void sdPrintMesh(sdMeshPtr m, sdOutputFormat format);

/* from "subdiv2.5D_lists.c" */
extern sdMeshPtr sdNewMesh(void);
extern void sdDestroyMesh(sdMeshPtr *m);

/* from "subdiv2.5D_txitxa.c" */
extern void sdSetEpsilon(sdMeshPtr m, float epsilon);
extern void sdSetProgressiveMode(sdMeshPtr m, sdBoolean progressive);
extern void sdSubdivTriangle(sdMeshPtr m, sdTrianglePtr t);
extern void sdSubdivAllLeavesOfTriangle(sdMeshPtr m, sdTrianglePtr t);
extern void sdSubdivAllLeafTrianglesOnce(sdMeshPtr m);
extern void sdSubdivAllLeafTrianglesUntilSmooth(sdMeshPtr m);
extern void sdMergeTriangle(sdMeshPtr m, sdTrianglePtr t);
extern void sdMergeAllLeavesOfTriangle(sdMeshPtr m, sdTrianglePtr t);
extern void sdMergeAllLeafTrianglesOnce(sdMeshPtr m);
extern void sdRecalculateAllVertexPositions(sdMeshPtr m, int levelStartReposn,
					    int levelStopDetails, int nbits,
					    sdBoolean recalcDetails);
extern void sdSPIHTcodec(sdMeshPtr m, sdBoolean animate);


#endif /* _INC_SUBDIV25D_ */

